//
//  CurrentInputController.m
//  CurrentInput
//
//  Created by Phidgets on 2015-07-30.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "DistanceSensorController.h"

@implementation DistanceSensorController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetDistanceSensor_delete(&ch);
        ch = nil;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetDistanceSensor_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDistanceSensor_setOnDistanceChangeHandler(ch, gotDistanceSensorData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetDistanceSensor_setOnSonarReflectionsUpdateHandler(ch, gotSonarReflectionsUpdateData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks

static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

void gotDistanceSensorData(PhidgetDistanceSensorHandle phid, void *context, uint32 distance){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDistanceSensorChangeHandler:)
                                           withObject:[NSNumber numberWithDouble:distance]
                                        waitUntilDone:NO];
}

void gotSonarReflectionsUpdateData(PhidgetDistanceSensorHandle phid, void *context, const uint32_t distances[8], const uint32_t amplitudes[8], uint32_t count){
    NSMutableArray *distanceArray = [[NSMutableArray alloc] init];
    NSMutableArray *amplitudesArray = [[NSMutableArray alloc] init];
    for(int i = 0; i < 8; i++){
        [distanceArray addObject: [NSNumber numberWithInt:distances[i]]];
        [amplitudesArray addObject:[NSNumber numberWithInt:amplitudes[i]]];
    }
    [(__bridge id)context performSelectorOnMainThread:@selector(onSonarReflectionsUpdateHandler:)
                                           withObject:[NSArray arrayWithObjects:distanceArray,amplitudesArray,[NSNumber numberWithInt:count], nil]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    Phidget_DeviceID deviceID;
    uint32 minDataInterval, maxDataInterval, dataInterval;
    uint32 minDistanceChangeTrigger,maxDistanceChangeTrigger, distanceChangeTrigger;
    
    //Get information from channel which will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetDistanceSensor_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetDistanceSensor_getMinDataInterval(ch, &minDataInterval);
    PhidgetDistanceSensor_getDataInterval(ch, &dataInterval);
    PhidgetDistanceSensor_getMinDistanceChangeTrigger(ch, &minDistanceChangeTrigger);
    PhidgetDistanceSensor_getMaxDistanceChangeTrigger(ch, &maxDistanceChangeTrigger);
    PhidgetDistanceSensor_getDistanceChangeTrigger(ch, &distanceChangeTrigger);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataBox setHidden:NO];
    [settingsBox setHidden:NO];
    
    if(deviceID == PHIDID_DST1200){
        int sonarQuietMode;
        PhidgetDistanceSensor_getSonarQuietMode(ch, &sonarQuietMode);
        [quietModeCheck setIntValue:sonarQuietMode];
        [reflectionsBox setHidden:NO];
    }
    
    //Configure data interval slider with information read from channel
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    
    //Configure trigger slider with information read from channel
    [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%d mm",minDistanceChangeTrigger]];
    [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%d mm",maxDistanceChangeTrigger]];
    [changeTriggerSlider setMaxValue:maxDistanceChangeTrigger];
    [changeTriggerSlider setMinValue:minDistanceChangeTrigger];
    [changeTriggerSlider setDoubleValue:distanceChangeTrigger];
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%d mm", distanceChangeTrigger]];
    
    //Adjusting view height
    NSRect frame = [mainWindow frame];
    if(![reflectionsBox isHidden])
        frame.size.height = [mainWindow maxSize].height;
    else
        frame.size.height += settingsBox.frame.size.height;
    
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [reflectionsBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDistanceSensorChangeHandler:(NSNumber *)distance{
    [currentText setStringValue:[NSString stringWithFormat:@"%d mm", distance.intValue]];
}

- (void)onSonarReflectionsUpdateHandler:(NSArray *)dataArray{
    NSMutableArray *distanceArray = [dataArray objectAtIndex:0];
    NSMutableArray *amplitudesArray = [dataArray objectAtIndex:1];
    int count = [[dataArray objectAtIndex:2] intValue];
    [countText setStringValue:[NSString stringWithFormat:@"%d",count]];
    for(NSView *view in reflectionsBox.subviews){
        for(NSTextField *field in view.subviews){
            if(field.tag < 8){
                if(field.tag < count)
                    [field setStringValue:[NSString stringWithFormat:@"%d", [distanceArray[field.tag] intValue]]];
                else
                    [field setStringValue:@""];
            }
            else if(field.tag < 16){
                if((field.tag-8) < count)
                    [field setStringValue:[NSString stringWithFormat:@"%d", [amplitudesArray[field.tag-8] intValue]]];
                else
                    [field setStringValue:@""];
            }
        }
    }
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}


#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetDistanceSensor_setDistanceChangeTrigger(ch, [changeTriggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%d mm", [changeTriggerSlider intValue]]];
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetDistanceSensor_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
}

- (IBAction)setQuietMode:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetDistanceSensor_setSonarQuietMode(ch, [quietModeCheck intValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
}

@end
